/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "chip.h"
#include "lang.h"

chip *chip_queue=NULL;
chip *chip_queue_first=NULL;
menu_pth *pth_qe_first, *pth_qe;
char first_run_qq=1;

void reset_queue_chip(void)
{
    chip_queue = chip_queue_first;
}

void register_chip(chip *new_chip)
{
    chip *tmp;


    if(!(new_chip->chip_name && new_chip->chip_path )){
	printf(BAD_CHIP_NAME);
	return;
    }
    
    if(first_run_qq){
	chip_queue = chip_queue_first = NULL;
	first_run_qq = 0;
    }
    if(!(tmp = (chip *)malloc(sizeof(chip)))){
	printf(MEMORY_ALLOCATION_ERROR_CHIP_QUEUE);
	exit (-1);
    }
    memcpy(tmp, new_chip, sizeof(chip));

    if(!(tmp->chip_path = (char *)malloc(sizeof(char) * (strlen(new_chip->chip_path) +1)))){
	printf(MEMORY_ALLOCATION_ERROR_CHIP_QUEUE);
	exit (-1);
    }
    strcpy(tmp->chip_path, new_chip->chip_path);

    if(!(tmp->chip_name = (char *)malloc(sizeof(char) * (strlen(new_chip->chip_name) +1)))){
	printf(MEMORY_ALLOCATION_ERROR_CHIP_QUEUE);
	exit (-1);
    }
    strcpy(tmp->chip_name, new_chip->chip_name);

    tmp->next = NULL;
    if(!chip_queue_first){
	chip_queue = chip_queue_first = tmp;
	return;
    }
    while(chip_queue->next) chip_queue = (chip*)chip_queue->next;
    chip_queue->next = tmp;
    chip_queue = tmp;
}

char find_chip(char *name)
{
    for(chip_queue = chip_queue_first; chip_queue; chip_queue = chip_queue->next )
	if(!strcmp(chip_queue->chip_name, name)) return 1;
    return 0;
}

chip *get_chip()
{ 
    static chip dummy;
    if(!chip_queue){
	memset(&dummy,0,sizeof(chip));
	dummy.chip_name = "---------";
	dummy.chip_path = "\0";
    }
    return chip_queue ? chip_queue : &dummy; 
}

char remove_chip(char *name)
{
    char t = 1;
    chip *tmp = chip_queue_first;
    chip_queue = chip_queue_first;    
    while(chip_queue->next && (t=strcmp(chip_queue->chip_name, name))){ 
	tmp = (chip *)chip_queue;
	chip_queue = (chip *)chip_queue->next;
    }
    if(t)
	return 0;
    tmp->next = chip_queue->next;
    free(chip_queue);
    chip_queue = chip_queue_first;
    return 1;
}

void remove_all_chip()
{
    chip *tmp;
    chip_queue = chip_queue_first;        
    while(chip_queue->next){
	tmp = chip_queue->next;
	free(chip_queue);
	chip_queue = tmp;
    }
    chip_queue_first = chip_queue = NULL;
}

char invoke_action_chip(int action)
{
    if(!chip_queue) return 0;    
    switch(action){
	case ACTION_READ	: chip_queue->read_chip(); break;
	case ACTION_READ_SIG	: chip_queue->read_sig_chip(); break;
	case ACTION_WRITE	: chip_queue->write_chip(); break;
	case ACTION_ERASE	: chip_queue->erase_chip(); break;
	case ACTION_LOCK	: chip_queue->lock_chip(); break;
	case ACTION_UNLOCK	: chip_queue->unlock_chip(); break;
	case ACTION_VERIFY	: chip_queue->verify_chip(); break;
	case ACTION_TEST	: chip_queue->test_chip(); break;
    }
    return 1;
}


chip *get_chip_qe(void)
{
    return chip_queue;
}

char next_chip(void)
{

//    if(!(chip_queue->next && chip_queue)) return 0;
    if(!chip_queue) return 0;
    chip_queue = (chip *)chip_queue->next; 
    return 1;
}

char chip_compare(char *name1, char *name2){
    char *tmp1,*tmp2;
    int  a,b;
    
    a = strlen(name1);
    b = strlen(name2);
    tmp1 = a > b ? name2 : name1;
    tmp2 = a > b ? name1 : name2;    
    a = a > b ? b : a;

    for(;a > 0; a--){
	if(*(tmp1 + a) > *(tmp2 + a)) return 1;
    }

    return 0;
}

void remove_path_qe()
{
    menu_pth *tmp;
    while(pth_qe_first){
	tmp = pth_qe_first->next;
	free(pth_qe_first->name);
	free(pth_qe_first);
	pth_qe_first = tmp;
    }
}

void add_new_path_qe(char *pth, void *wg)
{
    static char first_use=1;
    menu_pth *tmp;
    char *p;

    if(!(tmp = (menu_pth *)malloc(sizeof(menu_pth)))){
	perror(MEMORY_ALLOCATION_ERROR_CHIP_QUEUE);
	exit(-1);
    }

    if(!(p = (char *)malloc(sizeof(char) * strlen(pth)))){
	perror(MEMORY_ALLOCATION_ERROR_CHIP_QUEUE);
	exit(-1);
    }

    strcpy(p,pth);
    tmp->name = p;
    tmp->wg = wg;
    tmp->next = NULL;
    if(first_use){
	pth_qe_first = pth_qe = tmp;
	first_use = 0;
	atexit(remove_path_qe);
	return;
    }
    pth_qe = pth_qe_first;
    while(pth_qe->next) pth_qe = (menu_pth *)pth_qe->next;
    pth_qe->next = tmp;    
}

/* zwraca wskaznik dla sciezki, lub NULL jesli sciezki nie ma */
void *find_path_qe(char *path)
{
    for(pth_qe = pth_qe_first; pth_qe; pth_qe = pth_qe->next)
	if(!strcmp(path, pth_qe->name)) return pth_qe->wg; 
    return NULL;    
}

/******************************************************************/

char split_path_name(char *src, char **name)
{
    char *tmp;
    int i;

    i = strlen( src );
    if(!i) return 0; /*sciezka musi miec dlugosc */
    if(*src != '/') return 0; /* scieka ma sie zaczynac od korzenia */

    for(tmp = src + i - 1; *tmp != '/'; tmp--);
    *name = tmp + 1;
    *tmp = 0;

    return 1;
}

char *last_pth(char *pth)
{
    for(pth += strlen(pth); *pth!='/' ; pth--);
    return pth + 1;
}
