/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __chip_h__
#define __chip_h__

typedef struct{
    char *chip_path;		/* chip path format (eg "/27xx"), visible later in menu */
    char *chip_name;		/* chip name eg 2716 */
    int  sub_id;		/* ID chip in family */
    int	 dip_switch;		/* dip switch settings for PCB3 - 29xx bit + 12bit dip switch       */
    void *image_willem;		/* pointer to xpm image for willem - the picture at the right 	     */
    void *image_pcb;    	/* as above, but for PCB3 					     */
    int  dev_size;		/* allocation size for buffer					     */
    int  chip_offset;
    char *buffer;
    int  checksum;
    int	 img_will_idx;
    int	 img_pcb3_idx;    
    char vpp;			/* programing voltage - if fixed, otherway -1			     */
    char vcc;			/* supply voltage, comments as above				     */
    char vcp;			/* supply voltage during programming, comments as above		     */
    int  prog_time;		/* max programing time pulse [us]	*/
    void  **option;		/* miscalenous option that could be set by user */
    void (*autostart)(void);	/* invocated just after module selection */
    void (*read_chip)(void);    /* read chip content to buffer */
    void (*read_sig_chip)(void);/* read chip signature (if can't NULL) */
    void (*write_chip)(void);   /* write content of buffer to chip */
    void (*erase_chip)(void);	/* erase chip (EEPROM) */
    void (*lock_chip)(void);	/* set lock to avoid later reading content */
    void (*unlock_chip)(void);  /* unlock locked chip - ok it's just program possibility :) */
    void (*verify_chip)(void);  /* compare content of chip with content of buffer */
    void (*test_chip)(void);	/* Is the chip OK ? */
    void  *next;		/* pointer to next chip structure */
} chip;

typedef struct {
    char *name;
    void *wg;
    void *next;
}menu_pth;


#define ACTION_READ		1
#define ACTION_READ_SIG		2
#define ACTION_WRITE		3
#define ACTION_ERASE		4
#define ACTION_LOCK		5
#define ACTION_UNLOCK		6
#define ACTION_VERIFY		7
#define ACTION_TEST		8
#define ACTION_BLANK_TEST	9

extern char find_chip(char *name);
extern void register_chip(chip *new_chip);
extern char remove_chip(char *name);
extern char invoke_action_chip(int action);
extern void reset_queue_chip(void);
extern chip *get_chip_qe(void);
extern char next_chip(void);
extern void sort_chip_queue(void);
extern void remove_path_qe();
extern void add_new_path_qe(char *pth, void *wg);
extern void *find_path_qe(char *path);
extern char split_path_name(char *src, char **name);
extern char *last_pth(char *pth);
extern chip *get_chip(void);
#endif

