/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dlfcn.h>
#include <unistd.h>
#include <dirent.h>

#include "timer.h"
#include "gepro.h"
#include "files.h"
#include "buffer.h"
#include "gui.h"
#include "main.h"
#include "chip.h"
#include "lang.h"
#include "willem.h"

modules mdl;

void rem_all_modl(modules *m)
{
    while(m->first_modl){
	m->modl = m->first_modl->next;
	dlclose(m->first_modl->handler);	
	free(m->first_modl->mod_name);
	free(m->first_modl);
	m->first_modl = m->modl;
    }
    m->modl = NULL;
}

void add_modl(char *mod_name, modules *m)
{
    static char first_run=1;
    mod_list *tmp;
    
    if(!(tmp = (mod_list *) malloc(sizeof(mod_list)))){
	perror(ADD_MODULE_ERROR);
	return;
    }
    
    if(!(tmp->mod_name = (char *)malloc(sizeof(char) * strlen(mod_name) + sizeof(char)) )){
	perror(ADD_MODULE_ERROR);
	return;
    }

    strcpy(tmp->mod_name,mod_name);
    tmp->init_module = tmp->handler = NULL;
    tmp->next = NULL;

    if(first_run){
	first_run = 0;
	m->modl = m->first_modl = tmp;
	return;
    }
    
    for(m->modl = m->first_modl; m->modl->next; m->modl = m->modl->next);
    m->modl->next = tmp;
}

char test_hw(void *wg, void *dont_stop)
{
/*
    char ex;

    if(test_connection()){
        if(dont_stop)
            dialog_box("Test hardware","\n     Hardware present ",BUTTON_OK,NULL);
        ex = 0;		    	    
    }else{
        dialog_box("Error !!!","\n hardware error :  Check Power and\n connections",BUTTON_OK,NULL);
        ex = 1;	
    }
    return ex;
*/
return 0;
}

void load_plugins(modules *m)
{
    char *errmsg;

    for(m->modl = m->first_modl; m->modl; m->modl = m->modl->next){
	dlerror();
	if(!(m->modl->handler = dlopen(m->modl->mod_name, RTLD_LAZY))){
	    printf(LD_MODULE_ERROR_LD, dlerror());
	    continue;
	}
	dlerror();
    	m->modl->init_module = dlsym(m->modl->handler, "init_module");
    	if((errmsg = dlerror())){
	    printf(LD_MODULE_ERROR_SYM, errmsg);
	    continue;
	}
	m->modl->init_module();
    }
}

void get_mod_fnames(char *path, modules *m)
{
    char *old_path, *tmp;
    DIR *dir;
    struct dirent *dr;

    old_path = getcwd(NULL,-1);
    chdir(path);    
    if(!(dir = opendir("./"))){
	perror(CH_DIR_ERROR);
	exit(-1);
    }
    
    while((dr = readdir(dir))){
	for(tmp = dr->d_name + strlen(dr->d_name); tmp != dr->d_name && *tmp != '.'; tmp--);
	if( strcmp(tmp, MODULE_EXTENSION) ) continue;
	tmp = (char *)malloc(sizeof(char) * (strlen(dr->d_name) + 3));
	strcpy(tmp + 2, dr->d_name);
	*tmp = '.'; *(tmp + 1) = '/';
	add_modl(tmp, m);
	free(tmp);
    }
    closedir(dir);
    chdir(old_path);
    free(old_path);
}

int main(int argc, char **argv)
{
    memset(&PROGRAM_STATE,0,sizeof(PROGRAM_STATE));
    SET_STARTAP(1);
    SET_BUFFER(NULL);
    get_mod_fnames(MODULES_PATH, &mdl);
    menu_setup(&argc,&argv);
    load_plugins(&mdl);
    vpp_off();vcc_off();
    gui_run();
    return 0;
}
