/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "modules.h"
#include "93Cxx.xpms"

#define C46_SIZE 128
#define C56_SIZE 256
#define C66_SIZE 512

#define A46_ID	1
#define B46_ID	2
#define C56_ID	3
#define C66_ID	4

#define OP_READ  0x18

#define TI 20

#define EWEN_93		0x03
#define ERAL_93		0x02
#define WRAL_93		0x01
#define EWDS_93		0x00

#define TCR 1000
#define TCW 10000

#define PARM_9346	128,8,7
#define PARM_9356	256,8,8
#define PARM_9366	512,8,9


void reset_93_socket()
{
    set_cs_pin(0);
    set_clk_pin(0);
    set_di_pin(0);    
}

char conn_test()
{
    return 0;
}

void tic_93_out(char bool)
{
    set_clk_pin(0);
    set_di_pin(bool);
    us_delay(TI);
    set_clk_pin(1);
    us_delay(TI);
    set_clk_pin(0);
}

char tic_93_in()
{
    char bit;

    set_clk_pin(1);
    us_delay(TI/2);
    bit=get_do_pin() ? 1 : 0;
    us_delay(TI/2);
    set_clk_pin(0);
    us_delay(TI);
    return bit;
}

void start_93_seq()
{
    set_cs_pin(0);
    set_clk_pin(0);
    set_di_pin(0);    

    us_delay(TI);
    set_di_pin(1);
    us_delay(TI/2);
    set_cs_pin(1);        
    us_delay(TI/2);
    set_clk_pin(1);
    us_delay(TI);
    set_clk_pin(0);
}

void stop_93_seq()
{
    set_cs_pin(0);
    set_clk_pin(0);
    set_di_pin(0);    
    us_delay(TI);
}

void send_instruction(unsigned int instr, unsigned char bytes)
{
    for(;bytes; bytes--) tic_93_out( (instr >> (bytes-1)) & 0x01);
}

unsigned int recv_93_data(char bites)
{
    unsigned int data=0;
    
    set_di_pin(0);
    for(; bites; bites--) data |= tic_93_in() << (bites-1);
    return data;
}

void send_op_code(unsigned int opcode, char bits)
{
    start_93_seq();
    send_instruction(opcode << (bits - 2), 2 + bits);
    stop_93_seq();
}

/**************************************************************************************************/
void read_9346(int dev_size, char data_bits, char addr_bits)
{
    int i;
    unsigned int d;
    
    if(conn_test()) return;
    progress_bar_init("Odczyt pamieci",dev_size);

    reset_93_socket();
    vcc_on();
    for(i = 0; i < dev_size; ){
	start_93_seq();
	send_instruction(2,2);
	send_instruction(i,addr_bits);

	d=recv_93_data(data_bits);
	if(data_bits==16)
	    write_buffer(i++,(d >> 8) & 0xff);
	write_buffer(i++,d & 0xff);
	stop_93_seq();
	us_delay(TCR);
	progress_bar_set(i);
    }
    vcc_off();
    progress_bar_free();
    checksum();
    stat_rfsh();
}


void erase_9346(int dev_size, char data_bits, char addr_bits)
{
    if(conn_test()) return;

    reset_93_socket();
    vcc_on();
    send_op_code(EWEN_93, addr_bits);
    send_op_code(ERAL_93, addr_bits);
    send_op_code(EWDS_93, addr_bits);
    vcc_off();
    read_9346(dev_size, data_bits, addr_bits);
}

void prog_9346(int dev_size, char data_bits, char addr_bits)
{
    int i;

    if(conn_test()) return;
    progress_bar_init("Odczyt pamieci",dev_size);

    reset_93_socket();
    vcc_on();

    send_op_code(EWEN_93, addr_bits);

    for(i = 0; i < dev_size;){
	start_93_seq();
	send_instruction(1,2);
	send_instruction(i,addr_bits);
	send_instruction(read_buffer(i++),8);
if(addr_bits==16)
	send_instruction(read_buffer(i++),8);
	stop_93_seq();
	us_delay(TCW);
	progress_bar_set(i);
    }
    send_op_code(EWDS_93, addr_bits);
    vcc_off();
    progress_bar_free();
}


void erase_9346A(void){ erase_9346(PARM_9346); }
void prog_9346A(void){  prog_9346(PARM_9346);  }
void read_9346A(void){  read_9346(PARM_9346);  }

void erase_9356(void){ erase_9346(PARM_9356); }
void prog_9356(void){  prog_9346(PARM_9356);  }
void read_9356(void){  read_9346(PARM_9356);  }

void erase_9366(void){ erase_9346(PARM_9366); }
void prog_9366(void){  prog_9346(PARM_9366);  }
void read_9366(void){  read_9346(PARM_9366);  }

void init_module(void)
{
    int img1,img2;
    INIT_MODULE("Init 93Cxx module\n");

    img1 = register_image(d_93xx_willem_xpm);
    img2 = register_image(d_93xx_pcb3_xpm);
    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/Serial EEPROM/93Cxx","93C46", A46_ID, C46_SIZE);
    INIT_IMAGE_SET_IDX(img1,img2);
    D_FUNC_INIT_SET(read_9346A, prog_9346A, erase_9346A, NULL);
    D_REGISTER;

    D_DATA_INIT_SET("/Serial EEPROM/93Cxx","93LC46", A46_ID, C46_SIZE);
    INIT_IMAGE_SET_IDX(img1,img2);
    D_FUNC_INIT_SET(read_9346A, prog_9346A, erase_9346A, NULL);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/Serial EEPROM/93Cxx","93C56", C56_ID, C56_SIZE);
    INIT_IMAGE_SET_IDX(img1,img2);
    D_FUNC_INIT_SET(read_9356, prog_9356, erase_9356, NULL);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/Serial EEPROM/93Cxx","93C66", C66_ID, C66_SIZE);
    INIT_IMAGE_SET_IDX(img1,img2);
    D_FUNC_INIT_SET(read_9366, prog_9366, erase_9366, NULL);
    D_REGISTER;
}
