/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifndef __modules_h__
#define __modules_h__
#include <stdio.h>
#include "../img_idx.h"

typedef struct{
    char *chip_path;		/* chip path format (eg "/27xx"), visible later in menu */
    char *chip_name;		/* chip name eg 2716 */
    int  sub_id;		/* ID chip in family */
    int	 dip_switch;		/* dip switch settings for PCB3 - 29xx bit + 12bit dip switch       */
    void *image_willem;		/* pointer to xpm image for willem - the picture at the right 	     */
    void *image_pcb;    	/* as above, but for PCB3 					     */
    int  dev_size;		/* allocation size for buffer					     */
    int  chip_offset;
    char *buffer;
    int  checksum;
    int	 img_will_idx;
    int	 img_pcb3_idx;    
    char vpp;			/* programing voltage - if fixed, otherway -1			     */
    char vcc;			/* supply voltage, comments as above				     */
    char vcp;			/* supply voltage during programming, comments as above		     */
    int  prog_time;		/* max programing time pulse [us]	*/
    void  **option;		/* miscalenous option that could be set by user */
    void (*autostart)(void);	/* invocated just after module selection */
    void (*read_chip)(void);    /* read chip content to buffer */
    void (*read_sig_chip)(void);/* read chip signature (if can't NULL) */
    void (*write_chip)(void);   /* write content of buffer to chip */
    void (*erase_chip)(void);	/* erase chip (EEPROM) */
    void (*lock_chip)(void);	/* set lock to avoid later reading content */
    void (*unlock_chip)(void);  /* unlock locked chip - ok it's just program possibility :) */
    void (*verify_chip)(void);  /* compare content of chip with content of buffer */
    void (*test_chip)(void);	/* Is the chip OK ? */
    void  *next;		/* pointer to next structure */
} chip;


#define FO_RB_FIRST	1
#define FO_RB_NEXT	2
#define FO_LABEL	3
#define FO_CHECK	4
#define FO_ENTRY	5
#define FO_CPAGE	6

#define FO_NONE		0
#define FO_H_FIRST	1
#define FO_H_NEXT	2

#define INIT_MODULE(msg) \
    chip __init_struct__; \
    printf(msg); 

#define SET_DIPSW(a) __init_struct__.dip_switch = a;

#define INIT_DEFAULT_SET \
    __init_struct__.chip_name = NULL;\
    __init_struct__.sub_id = __init_struct__.dev_size = __init_struct__.dip_switch = 0; \
    __init_struct__.vpp = __init_struct__.vcc = __init_struct__.vcp = __init_struct__.prog_time = -1; \
    __init_struct__.image_willem = __init_struct__.image_pcb = __init_struct__.option = NULL; \
    __init_struct__.read_sig_chip = __init_struct__.lock_chip = __init_struct__.unlock_chip = NULL; \
    __init_struct__.test_chip  = __init_struct__.read_chip = NULL; \
    __init_struct__.write_chip = __init_struct__.erase_chip = __init_struct__.verify_chip = NULL;\
    __init_struct__.autostart = NULL;
    
#define D_TEST(test)	__init_struct__.test_chip  = test
#define D_REGISTER register_chip(&__init_struct__);

#define INIT_IMAGE_SET_IDX(willem,pcb3)\
    __init_struct__.img_will_idx = willem;\
    __init_struct__.img_pcb3_idx = pcb3;

#define SET_AUTOSTART(fnc);\
    __init_struct__.autostart = fnc

#define D_DATA_INIT_SET( path, name, id, size) \
    __init_struct__.chip_path = path; __init_struct__.chip_name = name; \
    __init_struct__.sub_id = id; __init_struct__.dev_size = size;

#define D_FUNC_INIT_SET(read,write,erase,verify) \
    __init_struct__.read_chip = read; __init_struct__.write_chip = write; \
    __init_struct__.erase_chip = erase; __init_struct__.verify_chip = verify;

#define D_XFUNC_INIT_SET(sig,test,lock,unlock) \
    __init_struct__.read_chip = sig; __init_struct__.write_chip = test; \
    __init_struct__.erase_chip = lock; __init_struct__.verify_chip = unlock;

#define D_DIP_INIT_SET(dip)     __init_struct__.erase_chip = dip;


/* buffer */
extern char write_buffer(unsigned int addr, unsigned char byte);
extern int  read_buffer(unsigned int addr);
extern void checksum(void);

/* gui */
extern char dialog_box(char *title, char *txt, char *bt1, char *bt2);
extern void fast_option_add(int, char *, int, void *, void *, char);
extern void adv_add_page(char *);
extern void adv_option_add(int, char *, int, void *, void *, char);
extern void progress_bar_init(char *title, long range);
extern char progress_bar_set(long value);
extern void progress_bar_free();
extern void stat_rfsh(void);
extern void register_chip(chip *new_chip);
extern char gui_cmp_pls(int a,int b);
extern int  register_image(char **);  /* rejestruje now piksmap, zwraca jej indeks */

/* timer */
extern void us_delay(int ticks);
extern void arw_delay(long ticks);

/*************************************************************************/
/* programmer */
extern void vcc_on();
extern void vcc_off();
extern void vpp_on();
extern void vpp_off();
extern void set_dip_sw(int dip); /* It's a dummy function (maybe for future version ?) */
extern void set_vpp_voltage(float voltage); /* It's a dummy function (maybe for future version ?) */

/* ZIF-32 */
extern void set_par_data_pin(char value);
extern void set_par_addr_pin(int addr);
extern void set_we_pin(char bool);
extern void set_oe_pin(char bool);
extern void set_ce_pin(char bool);
extern char get_par_data_pin(void);

/* serial 93Cxx & 25Cxx */
extern void set_cs_pin(char bool);
extern void set_clk_pin(char bool);
extern void set_di_pin(char bool);
extern char get_do_pin(void);

/* serial 24Cxx & pic */
#define set_scl_pin(val)	set_clk_pin(val)
#define set_sda_pin(val)	set_di_pin(!(val))
#define set_test_pin(val)	set_cs_pin(val)
#define get_sda_pin()		(!get_do_pin())

#define ms_delay(ms)	us_delay((ms)*1000)
#define set_data(value)	set_par_data_pin((value) & 0xff);
#define set_address(a)	set_par_addr_pin(a);

#define oe(state, delay) set_oe_pin(state); us_delay(delay);
#define ce(state, delay) set_ce_pin(state); us_delay(delay);

#define progress_loop(cn, cnini, rounds, title)		\
	for(cn = 0, progress_bar_init("Odczyt", rounds);\
	gui_cmp_pls(cn, rounds); cn++,progress_bar_set(cn))

#define finish_action()		set_oe_pin(0); set_ce_pin(0);vpp_off(); \
				vcc_off(); checksum(); stat_rfsh();
#define start_action(oe,ce)   \
    vpp_off(); vcc_on(); set_oe_pin(oe); set_ce_pin(ce); us_delay(5000);


#define copy_data_to_buffer(addr) write_buffer(addr, get_par_data_pin())
#define copy_data_from_buffer(addr) set_data(read_buffer(addr))

#define cmp_data_and_buffer_ploop(addr, error) \
	if(get_par_data_pin() != read_buffer(addr)){ error = 0; progress_bar_free(); break;}

#define  show_message(sw,title,msg_1,msg_2) dialog_box(title,sw ? msg_2:msg_1,"OK",NULL);




#endif
