/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
    Aby mc odczytywa zawarto PROM serii 74S474 w programatorze WILLEM V 3.1 20-3-2002 
    naley przeci ciek od wyprowadzenia 28 ZIF i poczy j jumperem pomidzy istniejce 
    poczenie, a VCC ZIF.
    To umoliwi obsug ukadw 24 PIN.
    
    Aby mc odczytywa zawarto wszystkich 74S475, (wyjcia OC), naley w lini danych
    13..16,17..21 ZIF wstawi drabink 10k do zasilania.
    To ma na celu moliwo obsugi ukadw z wyjciami typu Open Collector
*/

#include "modules.h"

#define ADDR_MASK_474	0x600
#define SIZE_PROM_474	0x200
void read_474()
{
    int addr;

    progress_bar_init("Odczyt PROM",SIZE_PROM_474);
    vpp_off();
    vcc_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(1000);
    for(addr=0; addr < SIZE_PROM_474; addr++){
	progress_bar_set(addr);
	set_oe_pin(1);
	set_ce_pin(1);
	set_par_addr_pin(addr | ADDR_MASK_474);	
	us_delay(200);
	write_buffer(addr,get_par_data_pin());
	us_delay(200);
    }
    vcc_off();
    progress_bar_free();    
    checksum();
    stat_rfsh();
}

void verify_474()
{
    int addr;

    progress_bar_init("Odczyt PROM",SIZE_PROM_474);
    vpp_off();
    vcc_on();
    set_par_addr_pin(0);
    set_par_data_pin(0);
    us_delay(1000);
    for(addr=0; addr < SIZE_PROM_474; addr++){
	progress_bar_set(addr);
	set_oe_pin(1);
	set_ce_pin(1);
	set_par_addr_pin(addr | ADDR_MASK_474);	
	us_delay(200);
	if(read_buffer(addr) != get_par_data_pin() ){


	}
	us_delay(200);
    }
    vcc_off();
    progress_bar_free();    
    checksum();
    stat_rfsh();
}

void autostart_474()
{

}

void autostart_475()
{
    fast_option_add(FO_LABEL,"Warning:\nOpen Collector output",0,NULL,NULL,0);
}

void init_module(void)
{
    INIT_MODULE("Init PROM module\n");


    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/PROM","74S474", 0, SIZE_PROM_474);
    INIT_IMAGE_SET_IDX(0,0);
    D_FUNC_INIT_SET(read_474, NULL, NULL, verify_474);
    SET_DIPSW(0x30); /* 7 i 8 zaczone */
    SET_AUTOSTART(autostart_474);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/PROM","74S475", 0, SIZE_PROM_474);
    INIT_IMAGE_SET_IDX(0,0);
    D_FUNC_INIT_SET(read_474, NULL, NULL, verify_474);
    SET_DIPSW(0x30); /* 7 i 8 zaczone */
    SET_AUTOSTART(autostart_475);
    D_REGISTER;

    INIT_DEFAULT_SET;
    D_DATA_INIT_SET("/PROM","KR556RT5", 0, SIZE_PROM_474);
    INIT_IMAGE_SET_IDX(0,0);
    D_FUNC_INIT_SET(read_474, NULL, NULL, verify_474);
    SET_DIPSW(0x30); /* 7 i 8 zaczone */
    SET_AUTOSTART(autostart_475);
    D_REGISTER;

}
