/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/io.h>

#include "parport.h"

#define PARPORT_VERSION	"lib parport version 0.0.1\n"

unsigned char port_xmask[3] = {0x00,0xc8,0x0b};
unsigned char port_mirror[3];
unsigned char port_init = 1;
int port_sel=0;
unsigned int base_port[3]={LPT_1_ADDR,LPT_2_ADDR,LPT_3_ADDR};

void reset_ports(void);

void init_ports()
{
#ifdef PRINT_VERSION
    static char first_run=1;
    
    if(first_run){
	printf(PARPORT_VERSION);
	first_run=0;
    }
#endif
    port_init = 0;
    if( ioperm(base_port[port_sel], 3, 1) ){
	perror("Init port error.\n");
	exit(1);
    }  
    port_mirror[0] = port_mirror[1] = port_mirror[2] = 0;      
    reset_ports();
}

void outportb(unsigned int addr, unsigned char data)
{
    if(port_init) init_ports();
    asm("outb %%al,%%dx"::"d"(addr), "a"(data));    
}

char inportb(unsigned int addr)
{ 
    unsigned char data;
    if(port_init) init_ports();
    asm("inb %%dx,%%al":"=a"(data):"d"(addr));
    return data;
}

void set_par_port(unsigned char port_idx, unsigned char data)
{
    if(port_idx > 2) return;
    port_mirror[port_idx] = data;
    outportb(base_port[port_sel] + port_idx, data ^ port_xmask[port_idx]);
}

unsigned char get_par_port(unsigned char port_idx)
{
    if(port_idx > 2) return 0;
    port_mirror[port_idx] = inportb(base_port[port_sel] + port_idx);
    return port_mirror[port_idx] ^ port_xmask[port_idx];
}

void reset_ports()
{
    set_par_port(PA,0);
    set_par_port(PC,0);
    port_mirror[PB] = 0;
}

void port_select(unsigned char port_id)
{
    if(port_id > 2) port_id = 0;
    port_sel = port_id;
    port_init = 1;
    init_ports();
}

void set_bit(unsigned char idx, unsigned char mask)
{
    set_par_port(idx, port_mirror[idx] | mask);
}

void clr_bit(unsigned char idx, unsigned char mask)
{
    set_par_port(idx, port_mirror[idx] & ~mask);
}

unsigned char get_bit(unsigned char idx, unsigned char mask)
{
    return get_par_port(idx) & mask;
}
