/* geepro - Willem eprom programmer for linux
 * Copyright (C) 2006 Krzysztof Komarnicki
 * Email: krzkomar@wp.pl
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. See the file COPYING. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <sched.h>
#include <sys/time.h>
#include "timer.h"

#define VERSION "lib timer ver 0.0.1\n"
#define CHECK(sts,msg) if((sts) == -1) { perror(msg); return -1; }

int set_priority(int policy, int priority, schd *sch)
{
    sch->schd_param.sched_priority = (priority == MAX_PRIORITY) ? sch->high_priority :
				(priority == MIN_PRIORITY) ? sch->low_priority : priority;
    CHECK(sched_setscheduler(sch->pid, policy, &sch->schd_param), "set sched_setscheduler()");
    return 0;
}

int new_priority(int priority, schd *sch)
{
    sch->new_priority = 1;
    return set_priority(sch->policy, priority, sch);
}

int get_maxmin_priority(schd *sch)
{
    CHECK((sch->high_priority = sched_get_priority_max(sch->policy)), "sched_getpriority_max()");
    CHECK((sch->low_priority  = sched_get_priority_min(sch->policy)), "sched_getpriority_min()");
    return (sch->high_priority == -1) ? sch->high_priority : sch->low_priority;
}

int store_actual_policy(schd *sch)
{
    sch->old_policy = sched_getscheduler(sch->pid);
    CHECK(sch->old_policy, "sched_getscheduler()");
    return sch->old_policy;
}

int store_actual_priority(schd *sch)
{
    CHECK(sched_getparam(0, &sch->schd_param), "sched_getparam()");
    sch->old_priority = sch->schd_param.sched_priority;
    return sch->old_priority;
}

int init_priority(int pid, int policy, schd *sch)
{
    int tmp;
    static char version = 1;
    sch->pid = pid;
    sch->policy = policy;
    sch->new_priority = 0;
    if(get_maxmin_priority(sch) == -1) return -1;
    if((tmp = store_actual_policy(sch)) != -1)
	tmp = store_actual_priority(sch);
#ifdef PRINT_VERSION
    if(version){
	printf(VERSION);
	version = 0;
    }
#endif
    return tmp;
}

int restore_priority(schd *sch)
{
    if(!sch->new_priority) return 0;
    sch->new_priority = 0;
    return set_priority(sch->old_policy, sch->old_priority, sch);
}

long get_time()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec * 1000000 + tv.tv_usec;
}

void us_delay(int delay)
{
    static schd sched;
    long start;
    static char flag = 0; 
    
    if(!flag){
	init_priority(MY_PID, SCHED_FIFO,&sched);
	flag = 1;
    }
    if(delay < 100000)
	new_priority(MAX_PRIORITY, &sched);
    start = get_time();
    while((start + delay) > get_time());
    restore_priority(&sched);
}

void arw_delay(long delay)
{
    int seconds = delay / 1000000;
    int usec    = delay % 1000000;
    for(; seconds; seconds--) us_delay(1000000);
    us_delay(usec);
}
